package calculate

import (
	"electricity_bill_calc/model/calculate"
	"electricity_bill_calc/repository"
	"fmt"
)

func MainCalculateProcess(rid string) {
	report, err := repository.ReportRepository.GetReportIndex(rid)
	if err != nil {
		fmt.Println("1", err.Error()+"指定报表不存在")
		return
	}

	reportSummary, err := repository.ReportRepository.RetrieveReportSummary(rid)
	if err != nil {
		fmt.Println("2", err.Error()+"指定报表的基本电量电费数据不存在")
		return
	}

	summary := calculate.FromReportSummary(reportSummary, report)

	periodStart := report.Period.SafeLower()
	periodEnd := report.Period.SafeUpper()

	meterDetails, err := repository.MeterRepository.AllUsedMetersInReport(report.Id)
	if err != nil {
		fmt.Println("3", err)
		return
	}

	meterRelations, err := repository.CalculateRepository.GetAllPoolingMeterRelations(report.Park, periodStart.Time)
	if err != nil {
		fmt.Println("4", err)
		return
	}
	_, err = CheckMeterArea(report, meterDetails)
	if err != nil {
		fmt.Println("5", err)
		return
	}

	// 寻找每一个商户的所有表计读数，然后对分配到各个商户的表计读数进行初步的计算.
	tenementReports, err := TenementMetersCalculate(report, periodStart.Time, periodEnd.Time, meterDetails, summary)
	if err != nil {
		fmt.Println("6", err)
		return
	}

	// 取得所有公摊表计的读数，以及公摊表计对应的分摊表计
	poolingMetersReports, err := PooledMetersCalculate(report, periodStart.Time, periodEnd.Time, meterDetails, summary)
	if err != nil {
		fmt.Println("7", err)
		return
	}

	// 获取所有的物业表计，然后对所有的物业表计电量进行计算。
	parkMetersReports, err := MetersParkCalculate(*report, periodStart.Time, periodEnd.Time, meterDetails, summary)
	if err != nil {
		fmt.Println("8", err)
		return
	}

	// 计算所有表计的总电量
	parkTotal := TotalConsumptionCalculate(tenementReports, summary)

	// 计算线损以及调整线损
	err = LossCalculate(report, &parkMetersReports, &parkTotal, &summary)
	if err != nil {
		fmt.Println("9", err)
		return
	}

	// 计算所有已经启用的商铺面积总和，仅计算所有未迁出的商户的所有表计对应的商铺面积。
	err = EnabledAreaCalculate(&tenementReports, &summary)
	if err != nil{
		fmt.Println("10",err)
		return
	}

	fmt.Println(meterRelations, poolingMetersReports, parkMetersReports, parkTotal)

}
